// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Login Dialog                                                     ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################                                 #include "system.h"


#include "textmonitor.h"
#include "sqlmessages.h"


#include <qapp.h>
#if QT_VERSION < 210
#error QT Version 2.1 or better required!!!
#endif
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>


#include "logindialog.moc"


// ###### Constructor #######################################################
LoginDialog::LoginDialog(const QString& defaultServer,
                         const QString& defaultDatabase,
                         const QString& defaultUser,
                         QWidget*       parent,
                         const char*    name)
   : QDialog(parent,name,TRUE)
{
   QGridLayout* topLayout = new QGridLayout(this,3,2,5);
   CHECK_PTR(topLayout);

   // ====== Label ==========================================================
   QPixmap pixmap;
   if(pixmap.load("Pixmaps/Schlssel.xpm") == false) {
      cerr << "WARNUNG: Pixmap Schlssel.xpm nicht gefunden!" << endl;
   }

   QLabel* icon = new QLabel(this);
   CHECK_PTR(this);
   icon->setPixmap(pixmap);
   icon->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
   topLayout->addWidget(icon,0,0);

   QLabel* title = new QLabel("Login",this);
   CHECK_PTR(title);
   QFont font("Helvetica", 20, QFont::Bold);
   title->setFont(font);
   title->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
   title->setMinimumSize(title->sizeHint());
   topLayout->addWidget(title,0,1);


   // ====== Eingabefelder ==================================================
   QWidget* inputWidget = new QWidget(this);
   CHECK_PTR(inputWidget);
   QGridLayout* inputLayout = new QGridLayout(inputWidget,4,2,5);
   CHECK_PTR(inputLayout);

   QLabel* label[4];
   for(cardinal i = 0;i < 4;i++) {
      label[i] = new QLabel(inputWidget);
      CHECK_PTR(label[i]);
      inputLayout->addWidget(label[i],i + 1,0);
      LineEdit[i] = new QLineEdit(inputWidget);
      CHECK_PTR(LineEdit[i]);
      inputLayout->addWidget(LineEdit[i],i + 1,1);
   }

   label[0]->setText("Server:");
   label[1]->setText("Datenbank:");
   label[2]->setText("Benutzer:");
   label[3]->setText("Passwort:");
   LineEdit[0]->setText(defaultServer);
   LineEdit[1]->setText(defaultDatabase);
   LineEdit[2]->setText(defaultUser);
   LineEdit[3]->setFocus();
   LineEdit[3]->setEchoMode(QLineEdit::Password);
   topLayout->addMultiCellWidget(inputWidget,1,1,0,1);

   QPushButton* okayButton = new QPushButton("Okay",this);
   CHECK_PTR(okayButton);
   topLayout->addWidget(okayButton,2,0);

   QPushButton* cancelButton = new QPushButton("Abbruch",this);
   CHECK_PTR(cancelButton);
   topLayout->addWidget(cancelButton,2,1);


   // ====== Qt-Signale verbinden ===========================================
   for(cardinal i = 0;i < 4;i++) {
      QObject::connect(LineEdit[i],SIGNAL(returnPressed()),this,SLOT(accept()));
   }
   QObject::connect(okayButton,SIGNAL(clicked()),this,SLOT(accept()));
   QObject::connect(cancelButton,SIGNAL(clicked()),this,SLOT(reject()));

   setCaption("Datenbank-Login");
}


// ###### Login durchfhren #################################################
PgDatabase* LoginDialog::login(const QString& defaultServer,
                               const QString& defaultDatabase,
                               const QString& defaultUser)
{
   PgDatabase* database = NULL;
   LoginDialog dialog(defaultServer,defaultDatabase,defaultUser);
   do {
      if(dialog.exec()) {
         QString info = "host="      + dialog.LineEdit[0]->text() +
                        " dbname="   + dialog.LineEdit[1]->text() +
                        " user="     + dialog.LineEdit[2]->text() +
                        " password=" + dialog.LineEdit[3]->text();

         database = new PgDatabase(info);
         CHECK_PTR(database);
         if(database->ConnectionBad()) {
            sqlWarning(&dialog,"Login war nicht mglich!\n\n" +
                       QString(database->ErrorMessage()));
            delete database;
            database = NULL;
         }
      }
      else {
         return(NULL);
      }
   } while(database == NULL);
   return(database);
}
