// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Prfungsamt-Client                                               ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "paclient.h"
#include "paviews.h"
#include "tableviewer.h"
#include "textmonitor.h"
#include "sqlconnectioninterface.h"
#include "sqlmessages.h"


#include <qapp.h>
#if QT_VERSION < 210
#error QT Version 2.1 or better required!!!
#endif
#include <qplatinumstyle.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtabwidget.h>
#include <qprogressdialog.h>
#include <qmessagebox.h>


#include "paclient.moc"


// ###### Constructor #######################################################
PAClient::PAClient(SQLConnectionInterface* connection,
                   TextMonitor*            monitor,
                   QWidget*                parent,
                   const char*             name)
   : QMainWindow(parent,name)
{
   Connection = connection;
   Monitor    = monitor;

   QTabWidget* tabWidget = new QTabWidget(this);
   CHECK_PTR(tabWidget);

   // ====== Studenten-Tabelle ==============================================
   TableViewer* studentenTable = new TableViewer(Connection,
                                        (const ViewTable*)&Studenten_ViewTable,
                                        TableViewer::TVM_EditMode,
                                        NULL, NULL, NULL, NULL,
                                        tabWidget);
   CHECK_PTR(studentenTable);
   tabWidget->addTab(studentenTable,
                     QIconSet(QPixmap(Studenten_ViewTable.PixmapName)),
                     "Studenten");
   studentenTable->loadTable();

   // ====== Prfer-Tabelle =================================================
   TableViewer* prueferTable = new TableViewer(Connection,
                                        (const ViewTable*)&Pruefer_ViewTable,
                                        TableViewer::TVM_EditMode,
                                        NULL, NULL, NULL, NULL,
                                        tabWidget);
   CHECK_PTR(prueferTable);
   tabWidget->addTab(prueferTable,
                     QIconSet(QPixmap(Pruefer_ViewTable.PixmapName)),
                     "Prfer");
   prueferTable->loadTable();


   // ====== Statistiken-Tabelle ============================================
   TableViewer* statistikenTable = new TableViewer(Connection,
                                              (const ViewTable*)&Statistiken_ViewTable,
                                              TableViewer::TVM_EditMode|TableViewer::TVM_ReadOnly,
                                              NULL, NULL, NULL, NULL,
                                              tabWidget);
   CHECK_PTR(statistikenTable);
   tabWidget->addTab(statistikenTable,
                     QIconSet(QPixmap(Statistiken_ViewTable.PixmapName)),
                     "Statistiken");
   statistikenTable->loadTable();


   // ====== Statistik-Tabelle ==============================================
   TableViewer* qualifikattypenTable = new TableViewer(Connection,
                                              (const ViewTable*)&Qualifikattypen_ViewTable,
                                              TableViewer::TVM_EditMode|TableViewer::TVM_ReadOnly,
                                              NULL, NULL, NULL, NULL,
                                              tabWidget);
   CHECK_PTR(qualifikattypenTable);
   tabWidget->addTab(qualifikattypenTable,
                     QIconSet(QPixmap(Qualifikattypen_ViewTable.PixmapName)),
                     "Qualifikattypen");
   qualifikattypenTable->loadTable();


   // ====== Prfungstypen-Tabelle ==========================================
   TableViewer* pruefungstypenTable = new TableViewer(Connection,
                                              (const ViewTable*)&Pruefungstypen_ViewTable,
                                              TableViewer::TVM_EditMode|TableViewer::TVM_ReadOnly,
                                              NULL, NULL, NULL, NULL,
                                              tabWidget);
   CHECK_PTR(pruefungstypenTable);
   tabWidget->addTab(pruefungstypenTable,
                     QIconSet(QPixmap(Pruefungstypen_ViewTable.PixmapName)),
                     "Prfungstypen");
   pruefungstypenTable->loadTable();


   // ====== Men ===========================================================
   QMenuBar* menu = menuBar();
   QPopupMenu* dateiMenu = new QPopupMenu(this);
   CHECK_PTR(dateiMenu);
   dateiMenu->insertItem("&Beenden",this,SLOT(quit()),CTRL+Key_Q);

   QPopupMenu* hilfeMenu = new QPopupMenu(this);
   CHECK_PTR(hilfeMenu);
   hilfeMenu->insertItem("&About",this,SLOT(information()),CTRL+Key_T);
   hilfeMenu->insertItem("&What's This?",this,SLOT(whatsThis()),Key_F12);

   QPopupMenu* databaseMenu = new QPopupMenu(this);
   CHECK_PTR(databaseMenu);
   databaseMenu->insertItem("Monitor &ffnen",this,SLOT(openMonitor()),Key_F10);
   databaseMenu->insertItem("Monitor schl&ieen",this,SLOT(closeMonitor()),Key_F11);
   databaseMenu->insertSeparator();
   databaseMenu->insertItem("Optimieren",this,SLOT(optimize()));

   menu->insertItem("&Datei",dateiMenu);
   menu->insertItem("&Datenbank",databaseMenu);
   menu->insertSeparator();
   menu->insertItem("&Hilfe",hilfeMenu);


   // ====== Central-Widget initialisieren ==================================
   setCentralWidget(tabWidget);
   setCaption("Prfungsamt Informatik");
   show();
}


// ###### Destructor ########################################################
PAClient::~PAClient()
{
}


// ###### Client beenden ####################################################
void PAClient::quit()
{
   QApplication::exit(0);
}


// ###### Optimieren aller Tabellen #########################################
void PAClient::optimize()
{
   try {
      if(Monitor) {
         Monitor->write();
      }
      QProgressDialog progress("Lade Tabelle: Beginne Transaktion", "Abbruch",
                               PADB_OptimizeTable.Tables, this, "progress", TRUE);
      progress.show();
      for(cardinal i = 0;i < PADB_OptimizeTable.Tables;i++) {
         progress.setLabelText("Optimiere Tabelle:\n" + QString(PADB_OptimizeTable.Table[i]));
         qApp->processEvents();
         if(progress.wasCancelled()) {
            break;
         }
         Connection->optimize(PADB_OptimizeTable.Table[i]);
         progress.setProgress(i);
      }
   }
   catch(SQLException e) {
      sqlError(this,e.toString());
   }
}




// ###### Informationsfenster darstellen ####################################
void PAClient::information()
{
   QMessageBox::information(this,
      "Prfungsamt-Datenbank",
      "Prfungsamt-Client - Version 1.0\n\n"
      "Copyright (C) 2000\n"
      "Thomas Dreibholz\n"
      "Dreibholz@bigfoot.com",
      "Okay");
}


// ###### SQL-Monitor anzeigen ##############################################
void PAClient::openMonitor()
{
   if(Monitor != NULL) {
      Monitor->show();
      Monitor->raise();
   }
}


// ###### SQL-Monitor schlieen #############################################
void PAClient::closeMonitor()
{
   if(Monitor != NULL) {
      Monitor->hide();
   }
}
