-- ##########################################################################
-- ####                                                                  ####
-- ####                         Prfungsamt-Projekt                      ####
-- ####                    ============================                  ####
-- ####                                                                  ####
-- #### Trigger                                                          ####
-- ####                                                                  ####
-- #### Version 1.00  --  12. Juli 2000                                  ####
-- ####                                                                  ####
-- #### Copyright (C) 2000  Thomas Dreibholz                             ####
-- ####                     Universitt Bonn                             ####
-- ####                     EMail: Dreibholz@bigfoot.com                 ####
-- ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
-- ####                                                                  ####
-- ##########################################################################


-- ##########################################################################
-- #### Diplomarbeiten - Trigger                                         ####
-- ##########################################################################

DROP FUNCTION trigger_diplarbeiten_before();
CREATE FUNCTION trigger_diplarbeiten_before() RETURNS OPAQUE AS '
   DECLARE
      professor RECORD;
      immatr    DATE;
      exmatr    DATE;
      anzahl    INTEGER;
   BEGIN
      -- ====== Student mu immatrikuliert sein =============================
      SELECT INTO immatr s.immat_datum
         FROM pers_studenten s
         WHERE s.stud_id = NEW.stud_id;
      SELECT INTO exmatr s.exmat_datum
         FROM pers_studenten s
         WHERE s.stud_id = NEW.stud_id;
      IF immatr > NEW.anmelde_datum THEN
         RAISE EXCEPTION ''Der Student ist zum Anmeldezeitpunkt noch nicht immatrikuliert!'';
      END IF;
      IF (exmatr IS NOT NULL) AND (NEW.anmelde_datum > exmatr) THEN
         RAISE EXCEPTION ''Der Student ist zum Anmeldezeitpunkt schon exmatrikuliert!'';
      END IF;
      IF (NEW.ausgabe_datum IS NOT NULL) THEN
         IF immatr > NEW.ausgabe_datum THEN
            RAISE EXCEPTION ''Der Student ist zum Ausgabedatum noch nicht immatrikuliert!'';
         END IF;
         IF (exmatr IS NOT NULL) AND (NEW.ausgabe_datum > exmatr) THEN
            RAISE EXCEPTION ''Der Student ist zum Ausgabedatum schon exmatrikuliert!'';
         END IF;
      END IF;
      IF (NEW.abgabe_datum IS NOT NULL) THEN
         IF (NEW.ausgabe_datum IS NULL) THEN
            RAISE EXCEPTION ''Abgegebene Diplomarbeit wurde nicht ausgegeben?!'';
         END IF;
         IF (NEW.ausgabe_datum > NEW.abgabe_datum) THEN
            RAISE EXCEPTION ''Abgabedatum vor Ausgabedatum?!'';
         END IF;
         IF immatr > NEW.abgabe_datum THEN
            RAISE EXCEPTION ''Der Student ist zum Abgabedatum noch nicht immatrikuliert!'';
         END IF;
         IF (exmatr IS NOT NULL) AND (NEW.abgabe_datum > exmatr) THEN
            RAISE EXCEPTION ''Der Student ist zum Abgabedatum schon exmatrikuliert!'';
         END IF;
      END IF;
      IF (NEW.abgabe_limit IS NOT NULL) THEN
         IF (NEW.abgabe_limit IS NULL) THEN
            RAISE EXCEPTION ''Abgabelimit fr nicht ausgegebene Diplomarbeit?!'';
         END IF;
         IF (NEW.abgabe_limit < NEW.abgabe_datum) THEN
            RAISE EXCEPTION ''Abgabelimit vor Ausgabedatum?!'';
         END IF;
         IF immatr > NEW.abgabe_limit THEN
            RAISE EXCEPTION ''Der Student ist zum Abgabelimit noch nicht immatrikuliert!'';
         END IF;
         IF (exmatr IS NOT NULL) AND (NEW.abgabe_limit > exmatr) THEN
            RAISE EXCEPTION ''Der Student ist zum Abgabelimit schon exmatrikuliert!'';
         END IF;
      END IF;

      -- ====== berprfen, ob gew. Prfer ein Professor ist ================
      SELECT INTO professor p.pers_id
         FROM pers_pruefer p
         WHERE p.pers_id = NEW.gew_pruefer_id AND
               p.Rang IN (''C1'',''C2'',''C3'',''C4'');
      IF NOT FOUND THEN
         RAISE EXCEPTION ''Der gewnschter Prfer ist kein Professor!'', NEW.gew_pruefer_id;
      END IF;

      -- ====== Genehmigte Prfungen: 1. Prfer mu Professor sein ==========
      IF NEW.status_id <> ''A'' THEN
         SELECT INTO anzahl COUNT(p.pers_id)
            FROM pers_pruefer p
            WHERE p.pers_id = NEW.pruefer1_id AND
                  p.Rang IN (''C1'',''C2'',''C3'',''C4'');
         IF NOT FOUND THEN
            RAISE EXCEPTION ''Der Prfer ist kein Professor!'';
         END IF;
      END IF;

      -- ====== Genehmigte Prfungen: 2. Prfer mu Professor sein ==========
      IF NEW.status_id <> ''A'' THEN
         SELECT INTO anzahl COUNT(p.pers_id)
            FROM pers_pruefer p
            WHERE p.pers_id = NEW.pruefer2_id AND
                  p.Rang IN (''C1'',''C2'',''C3'',''C4'');
         IF NOT FOUND THEN
            RAISE EXCEPTION ''Der Prfer ist kein Professor!'';
         END IF;
      END IF;

      RETURN NEW;
   END;
' LANGUAGE 'plpgsql';


DROP FUNCTION trigger_diplarbeiten_after();
CREATE FUNCTION trigger_diplarbeiten_after() RETURNS OPAQUE AS '
   DECLARE
      record1 RECORD;
   BEGIN
      -- ====== Mehrfach beantragte/genehmigte Diplomarbeiten des gleichen Typs ==
      SELECT INTO record1 COUNT(d.stud_id), d.status_id
         FROM uni_diplarbeit d
         WHERE d.stud_id = NEW.stud_id AND
               (d.status_id = ''A'' OR
                d.status_id = ''G'')
         GROUP BY d.status_id
         HAVING COUNT(d.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Diplomarbeit wurde mehrfach beantragt oder genehmigt?!'';
      END IF;

      -- ====== Mehr als zwei gewertete Diplomarbeit ========================
      SELECT INTO record1 COUNT(d.stud_id)
         FROM uni_diplarbeit d,
              typ_prfstatus t
         WHERE d.stud_id    = NEW.stud_id AND
               t.status_typ = d.status_id AND
               (t.status_gewertet_nichtbestanden = TRUE OR
                t.status_gewertet_bestanden      = TRUE)
         GROUP BY d.status_id
         HAVING COUNT(d.stud_id) > 2;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Diplomarbeit wurde mehr als zweimal gewertet!'';
      END IF;

      -- ====== Mehr als einmal bestandene Diplomarbeiten ===================
      SELECT INTO record1 COUNT(d.stud_id)
         FROM uni_diplarbeit d,
              typ_prfstatus t
         WHERE d.stud_id    = NEW.stud_id AND
               t.status_typ = d.status_id AND
               t.status_gewertet_bestanden = TRUE
         GROUP BY d.status_id
         HAVING COUNT(d.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Diplomarbeit wurde mehrmals bestanden!'';
      END IF;

      -- ====== Diplomarbeit wurde schon bestanden =====================================
      SELECT INTO record1 d1.stud_id, d1.anmelde_datum, d1.status_id
         FROM uni_diplarbeit d1
         WHERE d1.stud_id = NEW.stud_id AND
               EXISTS (SELECT d2.anmelde_datum
                       FROM uni_diplarbeit d2, typ_prfstatus t2
                       WHERE d2.stud_id = d1.stud_id                    AND
                             ((d2.anmelde_datum < d1.anmelde_datum)   OR
                              ((d2.ausgabe_datum IS NOT NULL) AND
                               (d1.ausgabe_datum IS NOT NULL) AND
                               (d2.ausgabe_datum < d1.ausgabe_datum)) OR
                              ((d2.abgabe_datum IS NOT NULL) AND
                               (d1.abgabe_datum IS NOT NULL) AND
                               (d2.abgabe_datum < d1.ausgabe_datum))  OR
                              ((d2.abgabe_limit IS NOT NULL) AND
                               (d1.abgabe_limit IS NOT NULL) AND
                               (d2.abgabe_limit < d1.abgabe_limit))
                             )                                          AND
                             t2.status_typ = d2.status_id               AND
                             t2.status_gewertet_bestanden = TRUE);
      IF FOUND THEN
         RAISE EXCEPTION ''Die Diplomarbeit ist bereits vorher bestanden worden!'';
      END IF;

      RETURN NEW;
   END;
' LANGUAGE 'plpgsql';


DROP TRIGGER trigger_diplarbeiten_before ON uni_diplarbeit;
CREATE TRIGGER trigger_diplarbeiten_before BEFORE INSERT OR UPDATE ON uni_diplarbeit
   FOR EACH ROW EXECUTE PROCEDURE trigger_diplarbeiten_before();

DROP TRIGGER trigger_diplarbeiten_after ON uni_diplarbeit;
CREATE TRIGGER trigger_diplarbeiten_after AFTER INSERT OR UPDATE ON uni_diplarbeit
   FOR EACH ROW EXECUTE PROCEDURE trigger_diplarbeiten_after();



-- ##########################################################################
-- #### Prfungen - Trigger                                              ####
-- ##########################################################################

-- Qualifikat- und Prfungsdatum berprfen:
-- (l.datum <= NEW.anmelde_datum OR l.datum IS NULL) AND
-- in IF-Abfrage hinzufgen!

DROP FUNCTION trigger_pruefungen_before();
CREATE FUNCTION trigger_pruefungen_before() RETURNS OPAQUE AS '
   DECLARE
      professor RECORD;
      immatr    DATE;
      exmatr    DATE;
      anzahl    INTEGER;
   BEGIN
      -- ====== Student mu immatrikuliert sein =============================
      SELECT INTO immatr s.immat_datum
         FROM pers_studenten s
         WHERE s.stud_id = NEW.stud_id;
      SELECT INTO exmatr s.exmat_datum
         FROM pers_studenten s
         WHERE s.stud_id = NEW.stud_id;
      IF immatr > NEW.anmelde_datum THEN
         RAISE EXCEPTION ''Der Student ist zum Anmeldezeitpunkt noch nicht immatrikuliert!'';
      END IF;
      IF (exmatr IS NOT NULL) AND (NEW.anmelde_datum > exmatr) THEN
         RAISE EXCEPTION ''Der Student ist zum Anmeldezeitpunkt schon exmatrikuliert!'';
      END IF;
      IF (NEW.datum IS NOT NULL) THEN
         IF immatr > NEW.datum THEN
            RAISE EXCEPTION ''Der Student ist zum Prfungsdatum noch nicht immatrikuliert!'';
         END IF;
         IF (exmatr IS NOT NULL) AND (NEW.datum > exmatr) THEN
            RAISE EXCEPTION ''Der Student ist zum Prfungsdatum schon exmatrikuliert!'';
         END IF;
      END IF;

      -- ====== berprfen, ob gew. Prfer ein Professor ist ================
      SELECT INTO professor p.pers_id
         FROM pers_pruefer p
         WHERE p.pers_id = NEW.gew_pruefer_id AND
               p.Rang IN (''C1'',''C2'',''C3'',''C4'');
      IF NOT FOUND THEN
         RAISE EXCEPTION ''Der gewnschter Prfer ist kein Professor!'';
      END IF;

      -- ====== Genehmigte Prfungen: Prfer mu Professor sein =============
      IF NEW.status_id <> ''A'' THEN
         SELECT INTO professor p.pers_id
            FROM pers_pruefer p
            WHERE p.pers_id = NEW.pruefer_id AND
                  p.Rang IN (''C1'',''C2'',''C3'',''C4'');
         IF NOT FOUND THEN
            RAISE EXCEPTION ''Der Prfer ist kein Professor!'';
         END IF;
      END IF;

      -- ====== Hochschulzugangsberechtigung prfen =========================
      SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
         FROM uni_qualifikate l
         WHERE l.stud_id = NEW.stud_id AND
               (l.qal_typ IN (''HZB''));
      IF anzahl < 1 THEN
         RAISE EXCEPTION ''Es fehlt die Hochschulzugangsberechtigung!'';
      END IF;

      -- ===== Vordiplom fr Hauptstudiumsprfungen prfen ==================
      IF NEW.typ_id IN (''HD-A'',''HD-B'',''HD-C'',''HD-NF'') THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''VDIPL''));
         IF anzahl < 1 THEN
            RAISE EXCEPTION ''Es fehlt das Vordiplom-Zeugnis!'';
         END IF;
      END IF;

      -- ====== Vordiplom Informatik I/II ===================================
      IF NEW.typ_id = ''VD-I/II'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''IF-I'',''IF-II''));
         IF anzahl < 2 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr VD-I/II sind unvollstndig!'';
         ELSE
            RETURN NEW;
         END IF;
      END IF;

      -- ====== Vordiplom Informatik III/IV =================================
      IF NEW.typ_id = ''VD-III/IV'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''IF-III'',''IF-IV''));
         IF anzahl < 1 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr VD-III/IV sind unvollstndig!'';
         ELSE
            SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
               FROM uni_qualifikate l
               WHERE l.stud_id = NEW.stud_id AND
                     (l.qal_typ IN (''IF-PS'',''IF-PP''));
            IF anzahl < 2 THEN
               RAISE EXCEPTION ''Die Leistungsnachweise fr VD-III/IV sind unvollstndig!'';
            ELSE
               RETURN NEW;
            END IF;
         END IF;
      END IF;

      -- ====== Vordiplom Mathematik ========================================
      IF NEW.typ_id = ''VD-MATH'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''M-IR-I'',''M-IR-II'',''M-LA-I'',''M-LA-II''));
         IF anzahl < 3 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr VD-MATH sind unvollstndig!'';
         ELSE
            RETURN NEW;
         END IF;
      END IF;

      -- ====== Vordiplom Nebenfach =========================================
      IF NEW.typ_id = ''VD-NF'' THEN
         RETURN NEW;
      END IF;

      -- ====== Diplom A ====================================================
      IF NEW.typ_id = ''HD-A'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''IF-SA''));
         IF anzahl < 1 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr HD-A sind unvollstndig!'';
         ELSE
            RETURN NEW;
         END IF;
      END IF;

      -- ====== Diplom B ====================================================
      IF NEW.typ_id = ''HD-B'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''IF-SB''));
         IF anzahl < 1 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr HD-B sind unvollstndig!'';
         ELSE
            RETURN NEW;
         END IF;
      END IF;

      -- ====== Diplom C ====================================================
      IF NEW.typ_id = ''HD-C'' THEN
         SELECT INTO anzahl COUNT(DISTINCT l.qal_typ)
            FROM uni_qualifikate l
            WHERE l.stud_id = NEW.stud_id AND
                  (l.qal_typ IN (''IF-PC'',''M-PM''));
         IF anzahl < 2 THEN
            RAISE EXCEPTION ''Die Leistungsnachweise fr HD-C sind unvollstndig!'';
         ELSE
            RETURN NEW;
         END IF;
      END IF;

      -- ====== Diplom Nebenfach ============================================
      IF NEW.typ_id = ''HD-NF'' THEN
         RETURN NEW;
      END IF;

      -- ====== Unbekannter Typ =============================================
      RAISE EXCEPTION ''Unbekannter Prfungstyp "%"?!'', NEW.typ_id;
   END;
' LANGUAGE 'plpgsql';


DROP FUNCTION trigger_pruefungen_after();
CREATE FUNCTION trigger_pruefungen_after() RETURNS OPAQUE AS '
   DECLARE
      record1 RECORD;
      anzahl INTEGER;
   BEGIN
      -- ====== Mehrfach beantragte/genehmigte Prfungen des gleichen Typs ==
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id, p.status_id
         FROM uni_pruefungen p
         WHERE p.stud_id = NEW.stud_id AND
               (p.status_id = ''A'' OR
                p.status_id = ''G'')
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Prfung "%" wurde mehrfach beantragt oder genehmigt?!'',NEW.typ_id;
      END IF;

      -- ====== Mehr als drei gewertete Prfungen ===========================
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id
         FROM uni_pruefungen p,
              typ_prfstatus t
         WHERE p.stud_id    = NEW.stud_id AND
               t.status_typ = p.status_id AND
               (t.status_gewertet_nichtbestanden = TRUE OR
                t.status_gewertet_bestanden      = TRUE)
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 3;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Prfung "%" wurde mehr als dreimal gewertet!'',NEW.typ_id;
      END IF;

      -- ====== Mehr als einmal bestandene Prfungen ========================
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id
         FROM uni_pruefungen p,
              typ_prfstatus t
         WHERE p.stud_id    = NEW.stud_id AND
               t.status_typ = p.status_id AND
               t.status_gewertet_bestanden = TRUE
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Die Prfung "%" wurde mehrmals bestanden!'',NEW.typ_id;
      END IF;

      -- ====== Prfung wurde schon bestanden =====================================
      SELECT INTO record1 p1.stud_id, p1.datum, p1.status_id
         FROM uni_pruefungen p1
         WHERE p1.stud_id = NEW.stud_id AND
               EXISTS (SELECT p2.datum
                       FROM uni_pruefungen p2, typ_prfstatus t2
                       WHERE p2.stud_id = p1.stud_id      AND
                             p2.typ_id  = p1.typ_id       AND
                             (p2.anmelde_datum < p1.anmelde_datum OR
                              ((p2.datum IS NOT NULL) AND
                               (p1.datum IS NOT NULL) AND
                               (p2.datum < p1.datum))
                             )                            AND
                             t2.status_typ = p2.status_id AND
                             t2.status_gewertet_bestanden = TRUE);
      IF FOUND THEN
         RAISE EXCEPTION ''Die Prfung "%" ist bereits vorher bestanden worden!'',NEW.typ_id;
      END IF;

      RETURN NEW;
   END;

' LANGUAGE 'plpgsql';


DROP TRIGGER trigger_pruefungen_before ON uni_pruefungen;
CREATE TRIGGER trigger_pruefungen_before BEFORE INSERT OR UPDATE ON uni_pruefungen
   FOR EACH ROW EXECUTE PROCEDURE trigger_pruefungen_before();

DROP TRIGGER trigger_pruefungen_after ON uni_pruefungen;
CREATE TRIGGER trigger_pruefungen_after AFTER INSERT OR UPDATE ON uni_pruefungen
   FOR EACH ROW EXECUTE PROCEDURE trigger_pruefungen_after();



-- ##########################################################################
-- #### Qualifikate - Trigger                                            ####
-- ##########################################################################

DROP FUNCTION trigger_qualifikate_after();
CREATE FUNCTION trigger_qualifikate_after() RETURNS OPAQUE AS '
   DECLARE
      record1 RECORD;
      anzahl INTEGER;
   BEGIN
      -- ====== Mehr als eine Hochschulzugangsberechtigung ==================
      SELECT INTO record1 q.stud_id, COUNT(q.qal_typ)
         FROM uni_qualifikate q
         WHERE q.stud_id = NEW.stud_id AND
               q.qal_typ = ''HZB''
         GROUP BY q.stud_id
         HAVING COUNT(q.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Mehr als eine Hochschulzugangsberechtigung!'';
      END IF;

      -- ====== Mehr als ein Vordiplom ======================================
      SELECT INTO record1 q.stud_id, COUNT(q.qal_typ)
         FROM uni_qualifikate q
         WHERE q.stud_id = NEW.stud_id AND
               q.qal_typ = ''VDIPL''
         GROUP BY q.stud_id
         HAVING COUNT(q.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Mehr als ein Vordiplom!'';
      END IF;

      -- ====== Mehr als ein Diplom =========================================
      SELECT INTO record1 q.stud_id, COUNT(q.qal_typ)
         FROM uni_qualifikate q
         WHERE q.stud_id = NEW.stud_id AND
               q.qal_typ = ''DIPLOM''
         GROUP BY q.stud_id
         HAVING COUNT(q.stud_id) > 1;
      IF FOUND THEN
         RAISE EXCEPTION ''Mehr als ein Diplom!'';
      END IF;

      RETURN NEW;
   END;
' LANGUAGE 'plpgsql';


DROP TRIGGER trigger_qualifikate_after ON uni_qualifikate;
CREATE TRIGGER trigger_qualifikate_after AFTER INSERT OR UPDATE ON uni_qualifikate
   FOR EACH ROW EXECUTE PROCEDURE trigger_qualifikate_after();
