// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Datenbank-Verbindung                                             ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "sqlconnection.h"


#include <qstring.h>


// ###### Constructor #######################################################
SQLConnection::SQLConnection(PgDatabase* database, SQLMonitorInterface* monitor)
{
   Database    = database;
   Monitor     = monitor;
   Transaction = false;
}


// ###### Befehl ausfhren ##################################################
bool SQLConnection::executeCommandOk(const char* command)
{
   if(Monitor != NULL) {
      Monitor->write(command);
   }
   return(Database->ExecCommandOk(command));
}


// ###### Befehl ausfhren ##################################################
bool SQLConnection::executeTuplesOk(const char* command)
{
   if(Monitor != NULL) {
      Monitor->write(command);
   }
   return(Database->ExecTuplesOk(command));
}


// ###### Befehl ausfhren ##################################################
void SQLConnection::execute(const char* command)
{
   if(Transaction) {
      if(!executeCommandOk(command)) {
         throw SQLException("Fehler bei Kommando-Ausfhrung!\n\n" + QString(Database->ErrorMessage()));
      }
   }
   else {
      throw SQLException("Transaktionsbeginn fehlt!");
   }
}


// ###### Transaktion beginnen ##############################################
void SQLConnection::beginTransaction()
{
   if(Monitor != NULL) {
      Monitor->write();
   }
   if(!executeCommandOk("BEGIN TRANSACTION")) {
      throw SQLException("Fehler bei Transaktionsbeginn!\n\n" + QString(Database->ErrorMessage()));
      return;
   }
   Transaction = true;

   // SQL92 besagt, da eine Transaktion defaultmig "serializable" sein mu.
   // Bei PostgreSQL 7.0 ist das Default jedoch "read commited".
   execute("SET TRANSACTION ISOLATION LEVEL SERIALIZABLE");

   // Datumsformat fr Deutschland einstellen.
   execute("SET DATESTYLE = GERMAN");
}


// ###### Transaktion mit Commit beenden ####################################
void SQLConnection::commitTransaction()
{
   Transaction = false;
   if(!executeCommandOk("COMMIT TRANSACTION")) {
      throw SQLException("Fehler bei Transaktionsende!\n\n" + QString(Database->ErrorMessage()));
      return;
   }
}


// ###### Transaktion mit Rollback beenden ##################################
void SQLConnection::rollbackTransaction()
{
   if(Transaction == true) {
      Transaction = false;
      executeCommandOk("ROLLBACK TRANSACTION");
   }
}


// ###### Cursor deklarieren ################################################
void SQLConnection::createCursor(const char* cursorName, const char* command)
{
   if(Transaction) {
      // ====== Cursor deklarieren =============================================
      if(!executeCommandOk("DECLARE " + QString(cursorName) +
                                  " CURSOR FOR " + QString(command))) {
         throw SQLException("Fehler bei Declare Cursor!\n\n" + QString(Database->ErrorMessage()));
      }
   }
   else {
      throw SQLException("Transaktionsbeginn fehlt!");
   }
}


// ###### Fetch ausfhren ###################################################
void SQLConnection::fetchCursor(const char* cursorName, const char* what)
{
   if(Transaction) {
      // ====== Cursor deklarieren =============================================
      if(!executeTuplesOk("FETCH " + QString(what) +
                          " IN " + QString(cursorName))) {
         throw SQLException("Fehler bei Fetch!\n\n" + QString(Database->ErrorMessage()));
      }
   }
   else {
      throw SQLException("Transaktionsbeginn fehlt!");
   }
}


// ###### Cursor schlieen ##################################################
void SQLConnection::deleteCursor(const char* cursorName)
{
   if(!executeCommandOk("CLOSE " + QString(cursorName))) {
      throw SQLException("Fehler beim Schlieen des Cursors!\n\n" +
                         QString(Database->ErrorMessage()));
   }
}


// ###### Anzahl der Tupel im Cursor zurckgeben ############################
cardinal SQLConnection::getTuples()
{
   return(Database->Tuples());
}


// ###### Anzahl der Attribute im Cursor zurckgeben ########################
cardinal SQLConnection::getFields()
{
   return(Database->Fields());
}


// ###### Attribut zurckgeben ##############################################
const char* SQLConnection::getField(cardinal col)
{
   return(Database->FieldName(col));
}


// ###### Wert zurckgeben ##################################################
const char* SQLConnection::getValue(cardinal row, cardinal col)
{
   return(Database->GetValue(row,col));
}


// ###### Optimieren ########################################################
void SQLConnection::optimize(const char* tableName)
{
   if(!executeCommandOk("VACUUM ANALYZE " + QString(tableName))) {
      throw SQLException("Fehler bei Optimieren!\n\n" +
                         QString(Database->ErrorMessage()));
   }
}


// ###### Monitor setzen ####################################################
void SQLConnection::setMonitor(SQLMonitorInterface* monitor)
{
   Monitor = monitor;
}


// ###### Monitor zurckgeben ###############################################
SQLMonitorInterface* SQLConnection::getMonitor() const
{
   return(Monitor);
}
