DROP   TABLE typ_prfstatus;
CREATE TABLE typ_prfstatus
(
    status_typ CHAR(8)             NOT NULL,
    status_txt CHAR(30)            NOT NULL,
    status_gewertet_bestanden      BOOLEAN NOT NULL,
    status_gewertet_nichtbestanden BOOLEAN NOT NULL,

    PRIMARY KEY     (status_typ)
);
INSERT INTO typ_prfstatus VALUES ('A',   'angemeldet',false,false);
INSERT INTO typ_prfstatus VALUES ('G',   'genehmigt',false,false);

INSERT INTO typ_prfstatus VALUES ('B',   'bestanden',true,false);
INSERT INTO typ_prfstatus VALUES ('NG',  'bestanden, aber nicht gewertet',false,false);
INSERT INTO typ_prfstatus VALUES ('NB',  'nicht bestanden',false,true);

INSERT INTO typ_prfstatus VALUES ('AE',  'ausgefallen, entschuldigt',false,false);
INSERT INTO typ_prfstatus VALUES ('ANE', 'ausgefallen, unentschuldigt',false,true);
INSERT INTO typ_prfstatus VALUES ('AG',  'ausgefallen, Grund n.s.z.V.',false,false);


DROP TABLE uni_pruefungen;
CREATE TABLE uni_pruefungen (
   stud_id   INTEGER,
   typ_id    CHAR(12),
   anmelde_datum     DATE DEFAULT NOW(),
   datum     DATE DEFAULT NOW(),
   versuch   INTEGER,
   zaehler   INTEGER,
   status_id CHAR(8),
   CONSTRAINT zuerst_anmelden
      CHECK (anmelde_datum <= datum),
   FOREIGN KEY(status_id) REFERENCES typ_prfstatus(status_typ),
   PRIMARY KEY(stud_id,typ_id,versuch,zaehler)
);

DROP FUNCTION trigger_pruefungen_after();
CREATE FUNCTION trigger_pruefungen_after() RETURNS OPAQUE AS '
   DECLARE
      record1 RECORD;
      anzahl INTEGER;
   BEGIN

      -- ====== Mehrfach beantragte/genehmigte Prfungen des gleichen Typs ==
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id, p.status_id
         FROM uni_pruefungen p
         WHERE p.stud_id = NEW.stud_id AND
               (p.status_id = ''A'' OR
                p.status_id = ''G'')
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 1;
      IF FOUND THEN
--         RAISE EXCEPTION ''Prfung "%" mehrfach beantragt oder genehmigt!'',NEW.typ_id;
      END IF;

      -- ====== Mehr als drei gewertete Prfungen ===========================
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id
         FROM uni_pruefungen p,
              typ_prfstatus t
         WHERE p.stud_id    = NEW.stud_id AND
               t.status_typ = p.status_id AND
               (t.status_gewertet_nichtbestanden = TRUE OR
                t.status_gewertet_bestanden      = TRUE)
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 3;
      IF FOUND THEN
--         RAISE EXCEPTION ''Prfung "%" mehr als dreimal gewertet!'',NEW.typ_id;
      END IF;

      -- ====== Mehr als einmal bestandene Prfungen ========================
      SELECT INTO record1 COUNT(p.stud_id), p.typ_id
         FROM uni_pruefungen p,
              typ_prfstatus t
         WHERE p.stud_id    = NEW.stud_id AND
               t.status_typ = p.status_id AND
               t.status_gewertet_bestanden = TRUE
         GROUP BY p.typ_id, p.status_id
         HAVING COUNT(p.stud_id) > 1;
      IF FOUND THEN
--         RAISE EXCEPTION ''Prfung "%" mehrmals bestanden!'',NEW.typ_id;
      END IF;

      -- ====== Prfung wurde schon bestanden =====================================
      SELECT INTO record1 p1.stud_id, p1.versuch, p1.zaehler, p1.datum, p1.status_id
         FROM uni_pruefungen p1
         WHERE p1.stud_id = NEW.stud_id AND
               EXISTS (SELECT p1.datum
                       FROM uni_pruefungen p2, typ_prfstatus t2
                       WHERE p2.stud_id = p1.stud_id      AND
                             p2.typ_id  = p1.typ_id       AND
                             (p2.anmelde_datum < p1.datum OR
                              p2.datum < p1.datum)        AND
                             t2.status_typ = p2.status_id AND
                             t2.status_gewertet_bestanden = TRUE);
      IF FOUND THEN
--         RAISE EXCEPTION ''Prfung "%" wurde schon bestanden!'',NEW.typ_id;
      END IF;


      RETURN NEW;
   END;

' LANGUAGE 'plpgsql';

-- DROP TRIGGER trigger_pruefungen_after ON uni_pruefungen;
CREATE TRIGGER trigger_pruefungen_after AFTER INSERT OR UPDATE ON uni_pruefungen
   FOR EACH ROW EXECUTE PROCEDURE trigger_pruefungen_after();


BEGIN;
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','01.02.2000',1,1,'A');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','02.02.2000',2,2,'G');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','03.02.2000',3,3,'A');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','03.04.2000',4,3,'G');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','05.02.2000',5,4,'NG');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','06.02.2000',6,5,'NB');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','07.02.2000',6,6,'NB');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','08.02.2000',6,7,'NB');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','09.02.2000',6,8,'NB');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','10.02.2000',6,9,'B');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','11.02.2000',6,19,'B');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','12.02.2000',7,21,'A');
INSERT INTO uni_pruefungen VALUES ('1234','HD-A','01.01.2000','13.02.2000',8,32,'G');
END;

-- ====== Mehrfach beantragte/genehmigte Prfungen des gleichen Typs ========
SELECT COUNT(p.stud_id), p.typ_id, p.status_id
   FROM uni_pruefungen p
   WHERE p.stud_id = '1234' AND
         (p.status_id = 'A' OR
          p.status_id = 'G')
   GROUP BY p.typ_id, p.status_id
   HAVING COUNT(p.stud_id) > 1;

-- ====== Mehr als drei gewertete Prfungen =================================
SELECT COUNT(p.stud_id), p.typ_id
   FROM uni_pruefungen p,
        typ_prfstatus t
   WHERE p.stud_id    = '1234'      AND
         t.status_typ = p.status_id AND
         (t.status_gewertet_nichtbestanden = TRUE OR
          t.status_gewertet_bestanden      = TRUE)
   GROUP BY p.typ_id, p.status_id
   HAVING COUNT(p.stud_id) > 3;

-- ====== Mehr als einmal bestandene Prfungen ==============================
SELECT COUNT(p.stud_id), p.typ_id
   FROM uni_pruefungen p,
        typ_prfstatus t
   WHERE p.stud_id    = '1234'      AND
         t.status_typ = p.status_id AND
         t.status_gewertet_bestanden = TRUE
   GROUP BY p.typ_id, p.status_id
   HAVING COUNT(p.stud_id) > 1;

-- ====== Mehr als dreimal nicht bestandene Prfungen =======================
SELECT COUNT(p.stud_id), p.typ_id
   FROM uni_pruefungen p,
        typ_prfstatus t
   WHERE p.stud_id    = '1234'      AND
         t.status_typ = p.status_id AND
         t.status_gewertet_nichtbestanden = TRUE
   GROUP BY p.typ_id, p.status_id
   HAVING COUNT(p.stud_id) > 3;

-- ====== Prfung wurde schon bestanden =====================================
SELECT p1.stud_id, p1.versuch, p1.zaehler, p1.datum, p1.status_id
   FROM uni_pruefungen p1
   WHERE p1.stud_id = '1234' AND
         EXISTS (SELECT p1.datum
                 FROM uni_pruefungen p2, typ_prfstatus t2
                 WHERE p2.stud_id = p1.stud_id      AND
                       p2.typ_id  = p1.typ_id       AND
                       (p2.anmelde_datum < p1.datum OR
                        p2.datum < p1.datum)        AND
                       t2.status_typ = p2.status_id AND
                       t2.status_gewertet_bestanden = TRUE);
